from django.contrib import admin
from django.core.exceptions import ValidationError
from django.forms.models import BaseInlineFormSet
from django.utils.html import format_html
from django.shortcuts import redirect

from ticketing.models import Commande
from .models import Evenement, EventPromotionCode, EventSpecialPrice


class EventSpecialPriceInlineFormSet(BaseInlineFormSet):
    def clean(self):
        super(EventSpecialPriceInlineFormSet, self).clean()
        set_union = set()
        for form in self.forms:
            if form.cleaned_data and not form.cleaned_data.get('DELETE', False):
                block_list = form.cleaned_data.get('place_type').values_list('nom', flat=True)
                if set_union.intersection(block_list):
                    raise ValidationError('Les groupes de types des places doivent être uniques')
                set_union = set_union.union(block_list)


class EventSpecialPriceInline(admin.TabularInline):
    model = EventSpecialPrice
    formset = EventSpecialPriceInlineFormSet
    extra = 0
    filter_horizontal = ('place_type',)

    def has_add_permission(self, request):
        return True

    can_delete = True


class EventPromotionCodeInline(admin.TabularInline):
    model = EventPromotionCode
    extra = 0

    def has_add_permission(self, request):
        return True

    can_delete = True


@admin.register(Evenement)
class EvenementAdmin(admin.ModelAdmin):
    exclude = ('active',)
    list_display = ['id', 'titre', 'create_invitations', 'artiste', 'places_reserved', 'date_heure_debut']

    def date_heure_debut(self, obj):
        return str(obj.date)+' '+str(obj.heure_début)

    def places_reserved(self, obj):
        return obj.get_sold_places_count()

    def create_invitations(self, obj):
        returned_html = '<br/>'
       # print(obj, obj.active)
        if not obj.active:
            url = f'{obj.id}{"/invitations/"}'
            returned_html = '<a style="text-decoration: underline;" href="%s">%s</a>' % (url, 'Générer des invitations')
        else:
            commande = Commande.objects.filter(evenement_id=obj.id, is_invitation=True).last()
            if commande:
                number_places = commande.places.count()
                url_invitation = f'{obj.id}{"/invitations/"}{commande.id}'
                returned_html = '<img src="/static/admin/img/icon-yes.svg" alt="True"/>' \
                                '<span class="number_invit"> %s </span>  ' \
                                '  <a href="%s"  target="_blank"> <img class="icon-custom" ' \
                                'src="/static/images/eye.svg" ' \
                                'alt="Télécharger"/></a>' % (number_places, url_invitation)
            else:
                url = f'{obj.id}{"/invitations/"}'
                returned_html = '<a style="text-decoration: underline;" href="%s">%s</a>' % (
                url, 'Générer des invitations')
        return format_html(returned_html)

    create_invitations.allow_tags = True
    create_invitations.short_description = 'Invitations'

    places_reserved.allow_tags = True
    places_reserved.short_description = 'Places réservés'

    date_heure_debut.allow_tags = True
    date_heure_debut.short_description = 'Date et heure début'

    list_display_links = ['id', 'titre']
    list_filter = ['category', 'tags']
    date_hierarchy = 'date'
    search_fields = ['titre', 'artiste__nom', 'artiste__prénom']
    filter_horizontal = ('tags', 'sponsors', 'partenaires')
    inlines = (EventSpecialPriceInline, EventPromotionCodeInline)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        context = extra_context or {}
        context['event_active'] = Evenement.objects.get(id=object_id).active
        return super(EvenementAdmin, self).changeform_view(request, object_id, form_url, extra_context=context)

    def add_view(self, request, form_url='', extra_context=None):
        context = extra_context or {}
        context['event_active'] = False
        return super(EvenementAdmin, self).changeform_view(request, None, form_url, extra_context=context)

    def response_add(self, request, obj, post_url_continue=None):
        return redirect('invitations-create', event_id=obj.id)

    def response_change(self, request, obj):
        if not obj.active:
            return redirect('invitations-create', event_id=obj.id)
        return redirect('/admin/calendrier/evenement/')
