from django.utils import timezone
from django.core.validators import MaxValueValidator
from json import dumps
from django.db import models, connection
from django.core.exceptions import ValidationError

from acteurs.models import Artiste, Sponsor, Partenaire
from meta.models import EvenementCategorie, Tag, PlaceType
from structure.models import Place


class Evenement(models.Model):
    titre = models.CharField('Nom de l’événement', max_length=150, unique=True)
    artiste = models.ForeignKey(Artiste, on_delete=models.CASCADE)
    date = models.DateField()
    heure_début = models.TimeField()
    heure_fin = models.TimeField(blank=True, null=True)
    prix_par_default = models.PositiveIntegerField('prix par défaut', default=1000)
    category = models.ForeignKey(EvenementCategorie, verbose_name='catégorie', on_delete=models.SET_NULL, null=True,
                                 blank=True)
    tags = models.ManyToManyField(Tag, verbose_name="mots clés", blank=True)
    sponsors = models.ManyToManyField(Sponsor, blank=True)
    partenaires = models.ManyToManyField(Partenaire, blank=True)
    date_de_début_de_vente = models.DateTimeField('date de début de la vente', blank=True, null=True)
    date_de_fin_de_vente = models.DateTimeField('date de fin de la vente', blank=True, null=True)
    active = models.BooleanField(default=False)
    couleur = models.CharField("Couleur de l'événement", max_length=150, null=True, default='#F8AB30')

    def clean(self):
        if not self.date_de_fin_de_vente.date() or not self.date_de_début_de_vente.date():
            raise ValidationError('Les dates et les heures de fin et de début de vente sont obligatoires')
        else:
            if self.date < self.date_de_fin_de_vente.date() \
                    or (self.date == self.date_de_fin_de_vente.date() and self.heure_début < (
                    self.date_de_fin_de_vente - timezone.timedelta(hours=3)).time()):
                raise ValidationError('Les date et heure de fin de vente ne peut dépasser'
                                      'les 3 heures après l\'heure de début de l\'événement')

    def get_status(self):
        today = timezone.localtime(timezone.now())
        if self.date_de_début_de_vente and today < self.date_de_début_de_vente:
            print('NOT_STARTED')
            return 'NOT_STARTED'
        if self.date_de_fin_de_vente and today > self.date_de_fin_de_vente:
            print('CLOSED')
            return 'CLOSED'
        if self.get_available_count() == 0:
            print('FULL')
            return 'FULL'
        if not self.active:
            return 'ONHOLD'
        print('OPEN')
        return 'OPEN'

    def get_sold_places_count(self):
        remain = Place.objects.all().count() - self.get_available_count()
        return remain

    def get_available_places(self):
        l = {}
        m = {}
        for place in Place.objects.raw("""
         select * from (select  structure_place.id, meta_placetype.nom
         from meta_placetype join structure_place on 
         structure_place.type_id = meta_placetype.id) as all_places 
          left join (select ticketing_reservation.place_id, orders.canceled_at, orders.confirmed_at, orders.paid_at
          from (select id, canceled_at, confirmed_at, paid_at from ticketing_commande 
         where ticketing_commande.evenement_id = {event_id})
           as  orders join ticketing_reservation on orders.id = ticketing_reservation.commande_id group by ticketing_reservation.place_id, orders.paid_at  HAVING not orders.paid_at ISNULL)
            reserved_places on all_places.id = reserved_places.place_id 
            where reserved_places.place_id ISNULL or not reserved_places.canceled_at ISNULL
            or (not reserved_places.confirmed_at ISNULL and  reserved_places.confirmed_at < '{now_before_15_mins}' and reserved_places.paid_at ISNULL);
         """.format(event_id=self.id,
                    now_before_15_mins=timezone.now().replace(tzinfo=None) - timezone.timedelta(minutes=15))):
            if place.nom in l:
                l[place.nom].append(place.id)
            else:
                l[place.nom] = [place.id]

        for block_name in l:
            m[block_name] = len(l[block_name])
        return l, m


    def get_rested(self):
        print(self.get_sold_places_count())
        return  Place.objects.all().count() - self.get_sold_places_count()

    def get_sold_places_count(self):
        return sum(1 for _ in Place.objects.raw("""
                                    SELECT all_places.id AS id
                                            FROM
                                              (SELECT structure_place.id AS id
                                               FROM structure_place
                                               LEFT JOIN meta_placetype ON structure_place.type_id = meta_placetype.id) AS all_places
                                            JOIN
                                              (SELECT reservs.place_id,
                                                      reservs.price,
                                                      reservs.promo_code_id,
                                                      orders.paid_at
                                               FROM
                                                 (SELECT *
                                                  FROM ticketing_commande
                                                  WHERE ticketing_commande.evenement_id = {event_id}
                                                    AND NOT ticketing_commande.paid_at ISNULL) AS orders
                                               JOIN
                                                 (SELECT *
                                                  FROM ticketing_reservation) reservs ON orders.id = reservs.commande_id
                                               GROUP BY reservs.place_id
                                               HAVING NOT orders.paid_at ISNULL) reservations ON all_places.id = reservations.place_id;
                                                """.format(event_id=self.id,
                               now_before_15_mins=timezone.now().replace(tzinfo=None) - timezone.timedelta(
                                   minutes=15))))


    def get_available_count(self):
        return sum(1 for _ in Place.objects.raw("""
                select all_places.id as id from
                  (select structure_place.id as id from structure_place
                    left join meta_placetype
                      on structure_place.type_id = meta_placetype.id)
                  as all_places
                  left join
                  (select ticketing_reservation.place_id, orders.canceled_at, orders.confirmed_at, orders.paid_at from
                    (select id, canceled_at, confirmed_at, paid_at from ticketing_commande where ticketing_commande.evenement_id = {event_id})
                      as  orders
                    join ticketing_reservation on orders.id = ticketing_reservation.commande_id group by ticketing_reservation.place_id, orders.paid_at HAVING not orders.paid_at ISNULL) reserved_places
                    on all_places.id = reserved_places.place_id
                    where reserved_places.place_id ISNULL or not reserved_places.canceled_at ISNULL
                   or ( not reserved_places.confirmed_at ISNULL and  reserved_places.confirmed_at < '{now_before_15_mins}' and reserved_places.paid_at ISNULL);
                    """.format(event_id=self.id,
                               now_before_15_mins=timezone.now().replace(tzinfo=None) - timezone.timedelta(
                                   minutes=15))))



    def get_available_count_price(self, price):
        places = Place.objects.raw("""SELECT all_places.id AS id
                                            FROM
                                              (SELECT structure_place.id AS id
                                               FROM structure_place
                                               LEFT JOIN meta_placetype ON structure_place.type_id = meta_placetype.id) AS all_places
                                            JOIN
                                              (SELECT reservs.place_id,
                                                      reservs.price,
                                                      reservs.promo_code_id,
                                                      orders.paid_at,
                                                      orders.canceled_at
                                               FROM
                                                 (SELECT *
                                                  FROM ticketing_commande
                                                  WHERE ticketing_commande.evenement_id = {event_id}
                                                    AND NOT ticketing_commande.paid_at ISNULL) AS orders
                                               JOIN
                                                 (SELECT *
                                                  FROM ticketing_reservation
                                                  WHERE ticketing_reservation.price = {price} AND ticketing_reservation.promo_code_id ISNULL ) reservs ON orders.id = reservs.commande_id
                                              ) reservations ON all_places.id = reservations.place_id  where not reservations.paid_at ISNULL and reservations.canceled_at ISNULL;
                      """.format(event_id=self.id, price=price))
        return sum(1 for _ in places)

    def get_special_prices(self):
        list_special_prices = {}
        default_blocks = list(PlaceType.objects.values_list('nom', flat=True))
        for special_price in self.eventspecialprice_set.all():
            list_special_prices[special_price.prix] = list(special_price.place_type.all().values_list('nom', flat=True))
        for price, blocks in list_special_prices.items():
            default_blocks = [x for x in default_blocks if x not in set(blocks)]
        if len(default_blocks) > 0:
            try:
                list_special_prices[self.prix_par_default].extend(default_blocks)
            except KeyError:
                list_special_prices[self.prix_par_default] = default_blocks
        list_special_prices = dict(sorted(list_special_prices.items(), key=lambda x: x[0], reverse=True))
        return list_special_prices

    def get_promo_codes(self):
        d = []
        for promo_code in self.eventpromotioncode_set.all():
            d.append({'code': promo_code.code,
                      'discount_percentage': promo_code.discount_percentage,
                      'max_usage_times': promo_code.max_usage_times,
                      'remaining_codes': promo_code.max_usage_times - promo_code.get_used_codes(),
                      'is_enabled': promo_code.is_enabled})
        return dumps(d)

    class Meta:
        verbose_name = "événement"

    def __str__(self):
        return "{}. {}".format(self.id, self.titre)


class EventSpecialPrice(models.Model):
    evenement = models.ForeignKey(Evenement, on_delete=models.CASCADE)
    place_type = models.ManyToManyField(PlaceType, verbose_name='zone')
    prix = models.PositiveIntegerField()

    class Meta:
        verbose_name = "classe de prix"
        verbose_name_plural = "classes de prix"


class EventPromotionCode(models.Model):
    evenement = models.ForeignKey(Evenement, on_delete=models.CASCADE)
    code = models.CharField('code promotionnel', help_text="Laisse vide pour générer le code promo automatiquement",
                            max_length=128, blank=True,
                            unique=True)
    discount_percentage = models.PositiveIntegerField('pourcentage de la réduction', default=10,
                                                      validators=[MaxValueValidator(100), ])
    max_usage_times = models.PositiveIntegerField('nombre d’utilisations maximum', default=10)
    is_enabled = models.BooleanField('activer ?', default=True)

    def get_used_codes(self):
        with connection.cursor() as cursor:
            cursor.execute("""SELECT COUNT(*)
                                    FROM calendrier_eventpromotioncode
                                           LEFT JOIN
                                             ( (SELECT * FROM ticketing_commande where ticketing_commande.evenement_id = {event_id}) AS orders
                                                 JOIN ticketing_reservation ON orders.id = ticketing_reservation.commande_id) AS reserved_places
                                            ON reserved_places.promo_code_id = calendrier_eventpromotioncode.id
                                    WHERE reserved_places.promo_code_id = {id}
                                      and reserved_places.canceled_at ISNULL
                                      and not reserved_places.confirmed_at ISNULL
                                      and (reserved_places.confirmed_at > '{now_before_15_mins}'
                                             or not reserved_places.paid_at ISNULL);""" .format(id=self.id,
                                                        event_id=self.evenement.id,
                                                        now_before_15_mins=timezone.now().replace(
                                                            tzinfo=None) - timezone.timedelta(
                                                            minutes=15)))
            return cursor.fetchone()[0]

    def __str__(self):
        return "{}".format(self.code)

    class Meta:
        verbose_name = "code promotionnel"
        verbose_name_plural = "codes promotionnels"
