import io

import qrcode
from django.contrib.auth.models import User
from django.contrib.postgres.fields import ArrayField
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.db import models
from TickIT.settings import MEDIA_URL
from acteurs.models import Client, Vendeur
from calendrier.models import Evenement, Tag, EventPromotionCode
from meta.models import CancelReason
from structure.models import Zone, Place
from ticketing.helpers import _createHash


class Commande(models.Model):
    evenement = models.ForeignKey(Evenement, verbose_name='événement', on_delete=models.SET_NULL, null=True)
    client = models.ForeignKey(Client, on_delete=models.SET_NULL, null=True, blank=True)
    places = models.ManyToManyField(Place, through='Reservation', related_name='commandes')
    vendeur = models.ForeignKey(Vendeur, on_delete=models.PROTECT, null=True, blank=True)
    created_at = models.DateTimeField('créé le', auto_now_add=True)
    updated_at = models.DateTimeField('changé le', auto_now=True)
    confirmed_at = models.DateTimeField('confirmé le', null=True, blank=True)
    paid_at = models.DateTimeField('payé le', null=True, blank=True)
    total_paid = models.FloatField('total payé', null=True, blank=True)
    canceled_at = models.DateTimeField('annulé le', null=True, blank=True)
    is_invitation = models.BooleanField('invitation', default=False)
    is_flagged = models.BooleanField('est signalée?', default=False)
    flag_reason = models.ForeignKey(CancelReason, verbose_name='raison de signaler', on_delete=models.PROTECT, null=True, blank=True)
    paimenType = models.CharField('paimenType', max_length=100, null=True, default=None)
    log = models.CharField('log', max_length=200, null=True, blank=True)

    def get_status(self):
        if self.canceled_at:
            return 'CANCELED'
        if self.paid_at:
            return 'PAID'
        if self.confirmed_at:
            return 'CONFIRMED'
        return 'NOT_CONFIRMED'

    def get_total(self):
        sum = 0.0
        for rsv in self.reservation.all():
            promo = 0.0
            if rsv.promo_code:
                promo = rsv.promo_code.discount_percentage
            sum = sum + rsv.price * (1 - promo * 0.01)
        return sum

    def __str__(self):
        return "Commande #{}".format(self.id)


class Reservation(models.Model):
    place = models.ForeignKey(Place, on_delete=models.CASCADE, related_name='reservation')
    commande = models.ForeignKey(Commande, on_delete=models.CASCADE, related_name='reservation')
    price = models.FloatField(null=True, blank=True)
    promo_code = models.ForeignKey(EventPromotionCode, null=True, on_delete=models.SET_NULL, blank=True)
    checkedin_at = models.DateTimeField(null=True, blank=True)
    hash = models.CharField(max_length=20, unique=True, editable=False)
    QR_code = models.ImageField(upload_to='reservation_qrcode', editable=False, null=True, blank=True)

    def price_with_promo(self):
        return self.price * (1 - self.promo_code.discount_percentage * 0.01)

    def qr_code_tag(self):
        return '<img src="%s%s" />' % (MEDIA_URL, self.QR_code)

    qr_code_tag.short_description = 'QR code'
    qr_code_tag.allow_tags = True

    def create_hash(self):
        try:
            if not self.hash:
                self.hash = _createHash()

            # if not self.QR_code:
            #     qr = qrcode.QRCode(
            #         version=1,
            #         error_correction=qrcode.constants.ERROR_CORRECT_M,
            #         box_size=6,
            #         border=2,
            #     )
            #     qr.add_data(self.hash)
            #     qr.make(fit=True)
            #
            #     img = qr.make_image(fill_color="green", back_color="white")
            #
            #     buffer = io.BytesIO()
            #     img.save(buffer)
            #     filename = '%s.png' % (self.hash)
            #     filebuffer = InMemoryUploadedFile(
            #         buffer, None, filename, 'image/png', buffer.tell(), None)
            #
            #     self.QR_code.save(filename, filebuffer)
        except:
            pass

    def save(self, *args, **kwargs):
        self.create_hash()
        super(Reservation, self).save()

    def __str__(self):
        return "Reservation #{}".format(self.id)
